﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;
using Verse.Sound;

namespace TROM
{
    public class GameCondition_ShellingNearby : GameCondition
    {
        private static readonly IntRange TicksBetweenStrikes = new IntRange(300, 900);

        private static readonly IntRange TicksBetweenAmbience = new IntRange(120, 600);

        public IntVec2 centerLocation = IntVec2.Invalid;

        public IntRange initialStrikeDelay = new IntRange(300, 600);

        public bool ambientSound;

        private int nextStrikeTicks;

        public int nextAmbientTicks = 30;

        private Sustainer soundSustainer;

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_Values.Look(ref centerLocation, "centerLocation");
            Scribe_Values.Look(ref nextStrikeTicks, "nextStrikeTicks", 0);
            Scribe_Values.Look(ref initialStrikeDelay, "initialStrikeDelay");
            Scribe_Values.Look(ref ambientSound, "ambientSound", defaultValue: false);
        }

        public override void Init()//done
        {
            base.Init();
            nextStrikeTicks = Find.TickManager.TicksGame + initialStrikeDelay.RandomInRange;
            nextAmbientTicks = Find.TickManager.TicksGame + TicksBetweenAmbience.RandomInRange;
            if (centerLocation.IsInvalid)
            {
                FindGoodCenterLocation();
            }
        }

        public override void GameConditionTick()
        {
            if (Find.TickManager.TicksGame > nextStrikeTicks)
            {
                //do stuff
                Shelling_HighExplosive b = (Shelling_HighExplosive)GenSpawn.Spawn(TROMThingDefOf.Shelling_HighExplosive, centerLocation.ToIntVec3, base.SingleMap);
                nextStrikeTicks = Find.TickManager.TicksGame + TicksBetweenStrikes.RandomInRange;
                FindGoodCenterLocation();
            }

            if (Find.TickManager.TicksGame > nextAmbientTicks)
            {
                TROMDef.Shelling_OffMap.PlayOneShot(new TargetInfo(Find.CameraDriver.MapPosition, base.SingleMap));
                nextAmbientTicks = Find.TickManager.TicksGame + TicksBetweenAmbience.RandomInRange;
            }
        }
        
        public override void End()
        {
            base.End();
        }

        private void FindGoodCenterLocation()//done
        {
            if (base.SingleMap.Size.x <= 16 || base.SingleMap.Size.z <= 16)
            {
                throw new Exception("Map too small for shelling.");
            }
            for (int i = 0; i < 10; i++)
            {
                centerLocation = new IntVec2(Rand.Range(8, base.SingleMap.Size.x - 8), Rand.Range(8, base.SingleMap.Size.z - 8));
            }
        }
    }
}
