﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using RimWorld;
using RimWorld.Planet;
using RimWorld.SketchGen;
using Verse;
using Verse.Noise;

namespace TROM
{
    class GenStep_TROM_ElevationFertility : GenStep
    {
        private const float ElevationFreq = 0.021f;
        private const float FertilityFreq = 0.021f;
        private const float EdgeMountainSpan = 0.42f;

        public override int SeedPart => 826504671;

        public override void Generate(Map map, GenStepParams parms)
        {
            //IntVec2 is 2d int array
            NoiseRenderer.renderSize = new IntVec2(map.Size.x, map.Size.z);
            //generate noise, put into array
            ModuleBase input = new Perlin(0.020999999716877937, 2.0, 0.5, 6, Rand.Range(0, int.MaxValue), QualityMode.High);
            //scale bias???
            input = new ScaleBias(0.5, 0.5, input);
            //saves raw noise for debugging
            NoiseDebugUI.StoreNoiseRender(input, "elev base");
            float num = 1f;
            //get hilliness factor
            switch (map.TileInfo.hilliness)
            {
                case Hilliness.Flat:
                    num = MapGenTuning.ElevationFactorFlat;
                    break;
                case Hilliness.SmallHills:
                    num = MapGenTuning.ElevationFactorSmallHills;
                    break;
                case Hilliness.LargeHills:
                    num = MapGenTuning.ElevationFactorLargeHills;
                    break;
                case Hilliness.Mountainous:
                    num = MapGenTuning.ElevationFactorMountains;
                    break;
                case Hilliness.Impassable:
                    num = MapGenTuning.ElevationFactorImpassableMountains;
                    break;
            }
            //scale noise to hilliness
            input = new Multiply(input, new Const(num));
            //saves factored noise for debugging
            NoiseDebugUI.StoreNoiseRender(input, "elev world-factored");
            //wtf???
            if (map.TileInfo.hilliness == Hilliness.Mountainous || map.TileInfo.hilliness == Hilliness.Impassable)
            {
                ModuleBase input2 = new DistFromAxis((float)map.Size.x * 0.42f);
                input2 = new Clamp(0.0, 1.0, input2);
                input2 = new Invert(input2);
                input2 = new ScaleBias(1.0, 1.0, input2);
                Rot4 random;
                do
                {
                    random = Rot4.Random;
                }
                while (random == Find.World.CoastDirectionAt(map.Tile));
                if (random == Rot4.North)
                {
                    input2 = new Rotate(0.0, 90.0, 0.0, input2);
                    input2 = new Translate(0.0, 0.0, -map.Size.z, input2);
                }
                else if (random == Rot4.East)
                {
                    input2 = new Translate(-map.Size.x, 0.0, 0.0, input2);
                }
                else if (random == Rot4.South)
                {
                    input2 = new Rotate(0.0, 90.0, 0.0, input2);
                }
                else
                {
                    _ = random == Rot4.West;
                }
                NoiseDebugUI.StoreNoiseRender(input2, "mountain");
                input = new Add(input, input2);
                NoiseDebugUI.StoreNoiseRender(input, "elev + mountain");
            }
            //maximum water coverage %
            float b = (map.TileInfo.WaterCovered ? 0f : float.MaxValue);
            //3d elevation array??? what does the 3rd dimension do
            MapGenFloatGrid elevation = MapGenerator.Elevation;
            //choose minimum elevation from factored noise and water level???
            foreach (IntVec3 allCell in map.AllCells)
            {
                elevation[allCell] = Mathf.Max(input.GetValue(allCell), 0.5f);
            }
            //fertility noise
            ModuleBase input3 = new Perlin(0.020999999716877937, 2.0, 0.5, 6, Rand.Range(0, int.MaxValue), QualityMode.High);
            //???
            input3 = new ScaleBias(0.5, 0.5, input3);
            //saves fertility noise for debugging
            NoiseDebugUI.StoreNoiseRender(input3, "noiseFert base");
            //3d fertility array??? what does the 3rd dimension do
            MapGenFloatGrid fertility = MapGenerator.Fertility;
            //set fertility level
            foreach (IntVec3 allCell2 in map.AllCells)
            {
                fertility[allCell2] = input3.GetValue(allCell2);
            }
        }
    }

    class GenStep_TROM_Terrain : GenStep
    {
        private struct GRLT_Entry
        {
            public float bestDistance;

            public IntVec3 bestNode;
        }

        private static bool debug_WarnedMissingTerrain = false;
        private static HashSet<IntVec3> tmpVisited = new HashSet<IntVec3>();
        private static List<IntVec3> tmpIsland = new List<IntVec3>();
        public override int SeedPart => 262606459;

        public override void Generate(Map map, GenStepParams parms)
        {
            BeachMaker.Init(map);
            RiverMaker riverMaker = GenerateRiver(map);
            List<IntVec3> list = new List<IntVec3>();
            MapGenFloatGrid elevation = MapGenerator.Elevation;
            MapGenFloatGrid fertility = MapGenerator.Fertility;
            TerrainGrid terrainGrid = map.terrainGrid;
            foreach (IntVec3 allCell in map.AllCells)
            {
                Building edifice = allCell.GetEdifice(map);
                TerrainDef terrainDef = null;
                terrainDef = TerrainFrom(allCell, map, elevation[allCell], fertility[allCell]);
                if (terrainDef.IsRiver && edifice != null)
                {
                    list.Add(edifice.Position);
                    edifice.Destroy();
                }
                terrainGrid.SetTerrain(allCell, terrainDef);
            }
            foreach (TerrainPatchMaker terrainPatchMaker in map.Biome.terrainPatchMakers)
            {
                terrainPatchMaker.Cleanup();
            }
        }

        private TerrainDef TerrainFrom(IntVec3 c, Map map, float elevation, float fertility)
        {
            TerrainDef terrainDef = null;
            TerrainDef terrainDef2 = null;
            if (terrainDef2 == TerrainDefOf.WaterOceanDeep)
            {
                return terrainDef2;
            }
            if (terrainDef != null && terrainDef.IsRiver)
            {
                return terrainDef;
            }
            if (terrainDef2 != null)
            {
                return terrainDef2;
            }
            if (terrainDef != null)
            {
                return terrainDef;
            }
            for (int i = 0; i < map.Biome.terrainPatchMakers.Count; i++)
            {
                terrainDef2 = map.Biome.terrainPatchMakers[i].TerrainAt(c, map, fertility);
                if (terrainDef2 != null)
                {
                    return terrainDef2;
                }
            }
            if (elevation > 0.55f && elevation < 0.61f)
            {
                return TerrainDefOf.Gravel;
            }
            if (elevation >= 0.61f)
            {
                return GenStep_RocksFromGrid.RockDefAt(c).building.naturalTerrain;
            }
            terrainDef2 = TerrainThreshold.TerrainAtValue(map.Biome.terrainsByFertility, fertility);
            if (terrainDef2 != null)
            {
                return terrainDef2;
            }
            if (!debug_WarnedMissingTerrain)
            {
                Log.Error("No terrain found in biome " + map.Biome.defName + " for elevation=" + elevation + ", fertility=" + fertility);
                debug_WarnedMissingTerrain = true;
            }
            return TerrainDefOf.Sand;
        }

        private void RemoveIslands(Map map)
        {
            CellRect mapRect = CellRect.WholeMap(map);
            int num = 0;
            tmpVisited.Clear();
            foreach (IntVec3 allCell in map.AllCells)
            {
                if (tmpVisited.Contains(allCell) || Impassable(allCell))
                {
                    continue;
                }
                int area = 0;
                bool touchesMapEdge2 = false;
                map.floodFiller.FloodFill(allCell, (IntVec3 x) => !Impassable(x), delegate (IntVec3 x)
                {
                    tmpVisited.Add(x);
                    area++;
                    if (mapRect.IsOnEdge(x))
                    {
                        touchesMapEdge2 = true;
                    }
                });
                if (touchesMapEdge2)
                {
                    num = Mathf.Max(num, area);
                }
            }
            if (num < 30)
            {
                return;
            }
            tmpVisited.Clear();
            foreach (IntVec3 allCell2 in map.AllCells)
            {
                if (tmpVisited.Contains(allCell2) || Impassable(allCell2))
                {
                    continue;
                }
                tmpIsland.Clear();
                TerrainDef adjacentImpassableTerrain = null;
                bool touchesMapEdge = false;
                map.floodFiller.FloodFill(allCell2, delegate (IntVec3 x)
                {
                    if (Impassable(x))
                    {
                        adjacentImpassableTerrain = x.GetTerrain(map);
                        return false;
                    }
                    return true;
                }, delegate (IntVec3 x)
                {
                    tmpVisited.Add(x);
                    tmpIsland.Add(x);
                    if (mapRect.IsOnEdge(x))
                    {
                        touchesMapEdge = true;
                    }
                });
                if ((tmpIsland.Count <= num / 20 || (!touchesMapEdge && tmpIsland.Count < num / 2)) && adjacentImpassableTerrain != null)
                {
                    for (int i = 0; i < tmpIsland.Count; i++)
                    {
                        map.terrainGrid.SetTerrain(tmpIsland[i], adjacentImpassableTerrain);
                    }
                }
            }
            bool Impassable(IntVec3 x)
            {
                return x.GetTerrain(map).passability == Traversability.Impassable;
            }
        }

        private RiverMaker GenerateRiver(Map map)
        {
            List<Tile.RiverLink> rivers = Find.WorldGrid[map.Tile].Rivers;
            if (rivers == null || rivers.Count == 0)
            {
                return null;
            }
            float angle = Find.WorldGrid.GetHeadingFromTo(map.Tile, rivers.OrderBy((Tile.RiverLink rl) => -rl.river.degradeThreshold).First().neighbor);
            Rot4 rot = Find.World.CoastDirectionAt(map.Tile);
            if (rot != Rot4.Invalid)
            {
                angle = rot.AsAngle + (float)Rand.RangeInclusive(-30, 30);
            }
            RiverMaker riverMaker = new RiverMaker(new Vector3(Rand.Range(0.3f, 0.7f) * (float)map.Size.x, 0f, Rand.Range(0.3f, 0.7f) * (float)map.Size.z), angle, rivers.OrderBy((Tile.RiverLink rl) => -rl.river.degradeThreshold).FirstOrDefault().river);
            GenerateRiverLookupTexture(map, riverMaker);
            return riverMaker;
        }

        private void UpdateRiverAnchorEntry(Dictionary<int, GRLT_Entry> entries, IntVec3 center, int entryId, float zValue)
        {
            float num = zValue - (float)entryId;
            if (!(num > 2f) && (!entries.ContainsKey(entryId) || entries[entryId].bestDistance > num))
            {
                entries[entryId] = new GRLT_Entry
                {
                    bestDistance = num,
                    bestNode = center
                };
            }
        }

        private void GenerateRiverLookupTexture(Map map, RiverMaker riverMaker)
        {
            int num = Mathf.CeilToInt(DefDatabase<RiverDef>.AllDefs.Select((RiverDef rd) => rd.widthOnMap / 2f + 8f).Max());
            int num2 = Mathf.Max(4, num) * 2;
            Dictionary<int, GRLT_Entry> dictionary = new Dictionary<int, GRLT_Entry>();
            Dictionary<int, GRLT_Entry> dictionary2 = new Dictionary<int, GRLT_Entry>();
            Dictionary<int, GRLT_Entry> dictionary3 = new Dictionary<int, GRLT_Entry>();
            for (int i = -num2; i < map.Size.z + num2; i++)
            {
                for (int j = -num2; j < map.Size.x + num2; j++)
                {
                    IntVec3 intVec = new IntVec3(j, 0, i);
                    Vector3 vector = riverMaker.WaterCoordinateAt(intVec);
                    int entryId = Mathf.FloorToInt(vector.z / 4f);
                    UpdateRiverAnchorEntry(dictionary, intVec, entryId, (vector.z + Mathf.Abs(vector.x)) / 4f);
                    UpdateRiverAnchorEntry(dictionary2, intVec, entryId, (vector.z + Mathf.Abs(vector.x - (float)num)) / 4f);
                    UpdateRiverAnchorEntry(dictionary3, intVec, entryId, (vector.z + Mathf.Abs(vector.x + (float)num)) / 4f);
                }
            }
            int num3 = Mathf.Max(dictionary.Keys.Min(), dictionary2.Keys.Min(), dictionary3.Keys.Min());
            int num4 = Mathf.Min(dictionary.Keys.Max(), dictionary2.Keys.Max(), dictionary3.Keys.Max());
            for (int k = num3; k < num4; k++)
            {
                WaterInfo waterInfo = map.waterInfo;
                if (dictionary2.ContainsKey(k) && dictionary2.ContainsKey(k + 1))
                {
                    waterInfo.riverDebugData.Add(dictionary2[k].bestNode.ToVector3Shifted());
                    waterInfo.riverDebugData.Add(dictionary2[k + 1].bestNode.ToVector3Shifted());
                }
                if (dictionary.ContainsKey(k) && dictionary.ContainsKey(k + 1))
                {
                    waterInfo.riverDebugData.Add(dictionary[k].bestNode.ToVector3Shifted());
                    waterInfo.riverDebugData.Add(dictionary[k + 1].bestNode.ToVector3Shifted());
                }
                if (dictionary3.ContainsKey(k) && dictionary3.ContainsKey(k + 1))
                {
                    waterInfo.riverDebugData.Add(dictionary3[k].bestNode.ToVector3Shifted());
                    waterInfo.riverDebugData.Add(dictionary3[k + 1].bestNode.ToVector3Shifted());
                }
                if (dictionary2.ContainsKey(k) && dictionary.ContainsKey(k))
                {
                    waterInfo.riverDebugData.Add(dictionary2[k].bestNode.ToVector3Shifted());
                    waterInfo.riverDebugData.Add(dictionary[k].bestNode.ToVector3Shifted());
                }
                if (dictionary.ContainsKey(k) && dictionary3.ContainsKey(k))
                {
                    waterInfo.riverDebugData.Add(dictionary[k].bestNode.ToVector3Shifted());
                    waterInfo.riverDebugData.Add(dictionary3[k].bestNode.ToVector3Shifted());
                }
            }
            CellRect cellRect = new CellRect(-2, -2, map.Size.x + 4, map.Size.z + 4);
            float[] array = new float[cellRect.Area * 2];
            int num5 = 0;
            for (int l = cellRect.minZ; l <= cellRect.maxZ; l++)
            {
                for (int m = cellRect.minX; m <= cellRect.maxX; m++)
                {
                    IntVec3 intVec2 = new IntVec3(m, 0, l);
                    bool flag = true;
                    for (int n = 0; n < GenAdj.AdjacentCellsAndInside.Length; n++)
                    {
                        if (riverMaker.TerrainAt(intVec2 + GenAdj.AdjacentCellsAndInside[n]) != null)
                        {
                            flag = false;
                            break;
                        }
                    }
                    if (!flag)
                    {
                        Vector2 p = intVec2.ToIntVec2.ToVector2();
                        int num6 = int.MinValue;
                        Vector2 vector2 = Vector2.zero;
                        for (int num7 = num3; num7 < num4; num7++)
                        {
                            if (dictionary2.ContainsKey(num7) && dictionary2.ContainsKey(num7 + 1) && dictionary.ContainsKey(num7) && dictionary.ContainsKey(num7 + 1) && dictionary3.ContainsKey(num7) && dictionary3.ContainsKey(num7 + 1))
                            {
                                Vector2 p2 = dictionary2[num7].bestNode.ToIntVec2.ToVector2();
                                Vector2 p3 = dictionary2[num7 + 1].bestNode.ToIntVec2.ToVector2();
                                Vector2 p4 = dictionary[num7].bestNode.ToIntVec2.ToVector2();
                                Vector2 p5 = dictionary[num7 + 1].bestNode.ToIntVec2.ToVector2();
                                Vector2 p6 = dictionary3[num7].bestNode.ToIntVec2.ToVector2();
                                Vector2 p7 = dictionary3[num7 + 1].bestNode.ToIntVec2.ToVector2();
                                Vector2 vector3 = GenGeo.InverseQuadBilinear(p, p4, p2, p5, p3);
                                if (vector3.x >= -0.0001f && vector3.x <= 1.0001f && vector3.y >= -0.0001f && vector3.y <= 1.0001f)
                                {
                                    vector2 = new Vector2((0f - vector3.x) * (float)num, (vector3.y + (float)num7) * 4f);
                                    num6 = num7;
                                    break;
                                }
                                Vector2 vector4 = GenGeo.InverseQuadBilinear(p, p4, p6, p5, p7);
                                if (vector4.x >= -0.0001f && vector4.x <= 1.0001f && vector4.y >= -0.0001f && vector4.y <= 1.0001f)
                                {
                                    vector2 = new Vector2(vector4.x * (float)num, (vector4.y + (float)num7) * 4f);
                                    num6 = num7;
                                    break;
                                }
                            }
                        }
                        if (num6 == int.MinValue)
                        {
                            Log.ErrorOnce("Failed to find all necessary river flow data", 5273133);
                        }
                        array[num5] = vector2.x;
                        array[num5 + 1] = vector2.y;
                    }
                    num5 += 2;
                }
            }
            float[] array2 = new float[cellRect.Area * 2];
            float[] array3 = new float[9] { 0.123317f, 0.123317f, 0.123317f, 0.123317f, 0.077847f, 0.077847f, 0.077847f, 0.077847f, 0.195346f };
            int num8 = 0;
            for (int num9 = cellRect.minZ; num9 <= cellRect.maxZ; num9++)
            {
                for (int num10 = cellRect.minX; num10 <= cellRect.maxX; num10++)
                {
                    IntVec3 intVec3 = new IntVec3(num10, 0, num9);
                    float num11 = 0f;
                    float num12 = 0f;
                    float num13 = 0f;
                    for (int num14 = 0; num14 < GenAdj.AdjacentCellsAndInside.Length; num14++)
                    {
                        IntVec3 c = intVec3 + GenAdj.AdjacentCellsAndInside[num14];
                        if (cellRect.Contains(c))
                        {
                            int num15 = num8 + (GenAdj.AdjacentCellsAndInside[num14].x + GenAdj.AdjacentCellsAndInside[num14].z * cellRect.Width) * 2;
                            if (array[num15] != 0f || array[num15 + 1] != 0f)
                            {
                                num11 += array[num15] * array3[num14];
                                num12 += array[num15 + 1] * array3[num14];
                                num13 += array3[num14];
                            }
                        }
                    }
                    if (num13 > 0f)
                    {
                        array2[num8] = num11 / num13;
                        array2[num8 + 1] = num12 / num13;
                    }
                    num8 += 2;
                }
            }
            array = array2;
            for (int num16 = 0; num16 < array.Length; num16 += 2)
            {
                if (array[num16] != 0f || array[num16 + 1] != 0f)
                {
                    Vector2 vector5 = Rand.InsideUnitCircle * 0.4f;
                    array[num16] += vector5.x;
                    array[num16 + 1] += vector5.y;
                }
            }
            byte[] array4 = new byte[array.Length * 4];
            Buffer.BlockCopy(array, 0, array4, 0, array.Length * 4);
            map.waterInfo.riverOffsetMap = array4;
            map.waterInfo.GenerateRiverFlowMap();
        }
    }

    class GenStep_TROM_Blocks : GenStep
    {
        public override int SeedPart => 738902354;

        private bool start = false, special = false, supermarket = false;

        private int blockSize, chunkSize, mapSize;

        int blockHeight, blockWidth;
        

        private enum blockType
        {
            Start,
            Residential
        };

        public override void Generate(Map map, GenStepParams parms)
        {
            blockSize = 38; 
            chunkSize = blockSize + 8;
            mapSize = map.Size.x;

            int cx = -36, blockx = 0;
            while (cx <= mapSize)
            {
                int cz = -36, blockz = 0;
                while (cz <= mapSize)
                {
                    bool done = false;
                    //decide on blocktype
                    if (blockx == 3 && blockz == 3)
                    {
                        GenerateBlock(map, blockType.Start, cx, cz);
                        done = true;
                    }
                    if (!done)
                    {
                        GenerateBlock(map, blockType.Residential, cx, cz);
                        done = true;
                    }

                    cz += chunkSize;
                    blockz += 1;
                }
                cx += chunkSize;
                blockx += 1;
            }
        }

        private void GenerateBlock(Map map, blockType bt, int cx, int cz)
        {
            CellRect blockRect = new CellRect(cx + 4, cz + 4, blockSize, blockSize);
            CellRect chunkRect = new CellRect(cx, cz, chunkSize, chunkSize);
            ResolveParams parms = default(ResolveParams);
            parms.sketch = new Sketch();
            parms.monumentSize = new IntVec2(blockSize, blockSize);

            switch (bt)
            {
                case blockType.Start:
                    GenerateBlockResidential(map, parms, cx, cz);
                    break;

                case blockType.Residential:
                    GenerateBlockResidential(map, parms, cx, cz);
                    break;

                default:
                    break;
            }
        }

        //blockType generation method dump
        private void GenerateBlockResidential(Map map, ResolveParams parms, int cx, int cz)
        {
            int bx = cx + 4;
            int bz = cz + 4;
            
            int nMin = 0, sMin = 0, eMin = 0, wMin = 0;
            int cyn = 11, cye = 11, cys = 11, cyw = 11;
            int nMax = 0, sMax = 0, eMax = 0, wMax = 0;
            int x, z;

            bool alleyway = Rand.Bool;
            bool alleywaye = false;
            int alleywaySide = Rand.Range(1, 5);

            IntVec2 intVec = parms.monumentSize.Value;
            blockWidth = intVec.x;//38
            blockHeight = intVec.z;//38
            nMax = blockHeight;
            sMax = blockHeight;
            eMax = blockWidth;
            wMax = blockWidth;
            Sketch block = new Sketch();
            bool onlyBuildableByPlayer = parms.onlyBuildableByPlayer ?? false;
            bool filterAllowsAll = parms.allowedMonumentThings == null;

            //resolve HouseCorner 9-11
            //SW
            ResolveParams parmsCSW = parms;
            int sw = Rand.Range(0, 100);
            //cut corner house
            if (sw <= 20)
            {
                parmsCSW.rect = new CellRect(0, 0, 0, 0);
                parmsCSW.sketch = new Sketch();
                SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R1, parmsCSW).Spawn(map, new IntVec3(bx, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                sMin = SketchResolver_HouseCorner_R1.x;
                wMin = SketchResolver_HouseCorner_R1.z;
            }
            //L-shaped house
            if (sw > 20 && sw <= 50)
            {
                parmsCSW.rect = new CellRect(0, 0, 0, 0);
                parmsCSW.sketch = new Sketch();
                SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R2, parmsCSW).Spawn(map, new IntVec3(bx, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                sMin = SketchResolver_HouseCorner_R2.x;
                wMin = SketchResolver_HouseCorner_R2.z;
            }
            //generic corner house
            if (sw > 50)
            {
                parmsCSW.rect = new CellRect(0, 0, 0, 0);
                parmsCSW.sketch = new Sketch();
                parmsCSW.monumentOpen = Rand.Bool;      //orientation
                SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_RG, parmsCSW).Spawn(map, new IntVec3(bx, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                if (!(bool)parmsCSW.monumentOpen)
                {
                    sMin = SketchResolver_HouseCorner_RG.x;
                    wMin = SketchResolver_HouseCorner_RG.z;
                }
                else
                {
                    sMin = SketchResolver_HouseCorner_RG.z;
                    wMin = SketchResolver_HouseCorner_RG.x;
                }
            }

            //SE
            ResolveParams parmsCSE = parms;
            int se = Rand.Range(0, 100);
            //cut corner house
            if (se <= 20)
            {
                parmsCSE.rect = new CellRect(0, 0, 0, 0);
                parmsCSE.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R1, parmsCSE);
                test.Rotate(Rot4.West);
                test.Spawn(map, new IntVec3(bx+blockWidth-1, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                eMin = SketchResolver_HouseCorner_R1.x;
                sMax -= SketchResolver_HouseCorner_R1.z;
            }
            //L-shaped house
            if (se > 20 && se <= 50)
            {
                parmsCSE.rect = new CellRect(0, 0, 0, 0);
                parmsCSE.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R2, parmsCSE);
                test.Rotate(Rot4.West);
                test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                eMin = SketchResolver_HouseCorner_R2.x;
                sMax -= SketchResolver_HouseCorner_R2.z;
            }
            //generic corner house
            if (se > 50)
            {
                parmsCSE.rect = new CellRect(0, 0, 0, 0);
                parmsCSE.sketch = new Sketch();
                parmsCSE.monumentOpen = Rand.Bool;      //orientation
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_RG, parmsCSE);
                test.Rotate(Rot4.West);
                test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                if (!(bool)parmsCSE.monumentOpen)
                {
                    eMin = SketchResolver_HouseCorner_RG.x;
                    sMax -= SketchResolver_HouseCorner_RG.z;
                }
                else
                {
                    eMin = SketchResolver_HouseCorner_RG.z;
                    sMax -= SketchResolver_HouseCorner_RG.x;
                }
            }

            //NW
            ResolveParams parmsCNW = parms;
            int nw = Rand.Range(0, 100);
            //cut corner house
            if (nw <= 20)
            {
                parmsCNW.rect = new CellRect(0, 0, 0, 0);
                parmsCNW.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R1, parmsCNW);
                test.Rotate(Rot4.East);
                test.Spawn(map, new IntVec3(bx, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                wMax -= SketchResolver_HouseCorner_R1.x;
                nMin = SketchResolver_HouseCorner_R1.z;
            }
            //L-shaped house
            if (nw > 20 && nw <= 50)
            {
                parmsCNW.rect = new CellRect(0, 0, 0, 0);
                parmsCNW.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R2, parmsCNW);
                test.Rotate(Rot4.East);
                test.Spawn(map, new IntVec3(bx, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                wMax -= SketchResolver_HouseCorner_R2.x;
                nMin = SketchResolver_HouseCorner_R2.z;
            }
            //generic corner house
            if (nw > 50)
            {
                parmsCNW.rect = new CellRect(0, 0, 0, 0);
                parmsCNW.sketch = new Sketch();
                parmsCNW.monumentOpen = Rand.Bool;      //orientation
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_RG, parmsCNW);
                test.Rotate(Rot4.East);
                test.Spawn(map, new IntVec3(bx, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                if (!(bool)parmsCNW.monumentOpen)
                {
                    wMax -= SketchResolver_HouseCorner_RG.x;
                    nMin = SketchResolver_HouseCorner_RG.z;
                }
                else
                {
                    wMax -= SketchResolver_HouseCorner_RG.z;
                    nMin = SketchResolver_HouseCorner_RG.x;
                }
            }

            //NE
            ResolveParams parmsCNE = parms;
            int ne = Rand.Range(0, 100);
            //cut corner house
            if (ne <= 20)
            {
                parmsCNE.rect = new CellRect(0, 0, 0, 0);
                parmsCNE.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R1, parmsCNE);
                test.Rotate(Rot4.South);
                test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                nMax -= SketchResolver_HouseCorner_R1.x;
                eMax -= SketchResolver_HouseCorner_R1.z;
            }
            //L-shaped house
            if (ne > 20 && ne <= 50)
            {
                parmsCNE.rect = new CellRect(0, 0, 0, 0);
                parmsCNE.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_R2, parmsCNE);
                test.Rotate(Rot4.South);
                test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                nMax -= SketchResolver_HouseCorner_R2.x;
                eMax -= SketchResolver_HouseCorner_R2.z;
            }
            //generic corner house
            if (ne > 50)
            {
                parmsCNE.rect = new CellRect(0, 0, 0, 0);
                parmsCNE.sketch = new Sketch();
                parmsCNE.monumentOpen = Rand.Bool;      //orientation
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseCorner_RG, parmsCNE);
                test.Rotate(Rot4.South);
                test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                if (!(bool)parmsCNE.monumentOpen)
                {
                    nMax -= SketchResolver_HouseCorner_RG.x;
                    eMax -= SketchResolver_HouseCorner_RG.z;
                }
                else
                {
                    nMax -= SketchResolver_HouseCorner_RG.z;
                    eMax -= SketchResolver_HouseCorner_RG.x;
                }
            }

            //resolve HouseEdge 16-20
            //S
            while (sMax - sMin > 0)
            {
                ResolveParams parmsES = parms;

                if (alleyway && !alleywaye && alleywaySide == 1 && (sMin > 11 && sMin < 26))
                {
                    x = 2;
                    z = 7;
                    parmsES.rect = new CellRect(0, 0, x, z);
                    parmsES.sketch = new Sketch();
                    parmsES.SetCustom("rotation", Rot4.West);
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseAlleyway, parmsES);
                    test.Spawn(map, new IntVec3(bx + sMin, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    sMin += x;
                    alleywaye = true;
                }
                else if (sMax - sMin <= 11)
                {
                    x = sMax - sMin;
                    z = Rand.Range(7, 10);
                    parmsES.rect = new CellRect(0, 0, x, z);
                    parmsES.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsES);
                    test.Spawn(map, new IntVec3(bx + sMin, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    sMin += x;
                }
                else
                {
                    x = Rand.Range(6, 12);
                    z = Rand.Range(7, 10); ;
                    if (sMax - sMin - x < 6) x = sMax - sMin - 6;
                    parmsES.rect = new CellRect(0, 0, x, z);
                    parmsES.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsES);
                    test.Spawn(map, new IntVec3(bx + sMin, 0, bz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    sMin += x;
                }
            }
            //W
            while (wMax - wMin > 0)
            {
                ResolveParams parmsEW = parms;
                if (alleyway && !alleywaye && alleywaySide == 2 && (wMin > 11 && wMin < 26))
                {
                    x = 2;
                    z = 7;
                    parmsEW.rect = new CellRect(0, 0, x, z);
                    parmsEW.sketch = new Sketch();
                    parmsEW.SetCustom("rotation", Rot4.North);
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseAlleyway, parmsEW);
                    test.Rotate(Rot4.East);
                    test.Spawn(map, new IntVec3(bx, 0, bz + wMin + x - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    wMin += x;
                    alleywaye = true;
                }
                else if (wMax - wMin <= 11)
                {
                    x = wMax - wMin;
                    z = Rand.Range(7, 10); ;
                    parmsEW.rect = new CellRect(0, 0, x, z);
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEW);
                    test.Rotate(Rot4.East);
                    test.Spawn(map, new IntVec3(bx, 0, bz + wMin+x-1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    wMin += x;
                }
                else
                {
                    x = Rand.Range(6, 12);
                    z = Rand.Range(7, 10); ;
                    if (wMax - wMin - x < 6) x = wMax - wMin - 6;
                    parmsEW.rect = new CellRect(0, 0, x, z);
                    parmsEW.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEW);
                    test.Rotate(Rot4.East);
                    test.Spawn(map, new IntVec3(bx, 0, bz + wMin+x-1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    wMin += x;
                }
            }
            //N
            while (nMax - nMin > 0)
            {
                ResolveParams parmsEN = parms;
                if (alleyway && !alleywaye && alleywaySide == 3 && (nMin > 11 && nMin < 26))
                {
                    x = 2;
                    z = 7;
                    parmsEN.rect = new CellRect(0, 0, x, z);
                    parmsEN.sketch = new Sketch();
                    parmsEN.SetCustom("rotation", Rot4.East);
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseAlleyway, parmsEN);
                    test.Rotate(Rot4.South);
                    test.Spawn(map, new IntVec3(bx + nMin + x - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    nMin += x;
                    alleywaye = true;
                }
                else if (nMax - nMin <= 11)
                {
                    x = nMax - nMin;
                    z = Rand.Range(7, 10); ;
                    parmsEN.rect = new CellRect(0, 0, x, z);
                    parmsEN.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEN);
                    test.Rotate(Rot4.South);
                    test.Spawn(map, new IntVec3(bx + nMin + x - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    nMin += x;
                }
                else
                {
                    x = Rand.Range(6, 12);
                    z = Rand.Range(7, 10); ;
                    if (nMax - nMin - x < 6) x = nMax - nMin - 6;
                    parmsEN.rect = new CellRect(0, 0, x, z);
                    parmsEN.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEN);
                    test.Rotate(Rot4.South);
                    test.Spawn(map, new IntVec3(bx + nMin + x - 1, 0, bz + blockWidth - 1), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    nMin += x;
                }
            }
            //E
            while (eMax - eMin > 0)
            {
                ResolveParams parmsEE = parms;
                if (alleyway && !alleywaye && alleywaySide == 4 && (eMin > 11 && eMin < 26))
                {
                    x = 2;
                    z = 7;
                    parmsEE.rect = new CellRect(0, 0, x, z);
                    parmsEE.sketch = new Sketch();
                    parmsEE.SetCustom("rotation", Rot4.South);
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseAlleyway, parmsEE);
                    test.Rotate(Rot4.West);
                    test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + eMin), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    eMin += x;
                    alleywaye = true;
                }
                else if (eMax - eMin <= 11)
                {
                    x = eMax - eMin;
                    z = Rand.Range(7, 10); ;
                    parmsEE.rect = new CellRect(0, 0, x, z);
                    parmsEE.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEE);
                    test.Rotate(Rot4.West);
                    test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + eMin), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    eMin += x;
                }
                else
                {
                    x = Rand.Range(6, 12);
                    z = Rand.Range(7, 10); ;
                    if (eMax - eMin - x < 6) x = eMax - eMin - 6;
                    parmsEE.rect = new CellRect(0, 0, x, z);
                    parmsEE.sketch = new Sketch();
                    Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.HouseEdge, parmsEE);
                    test.Rotate(Rot4.West);
                    test.Spawn(map, new IntVec3(bx + blockWidth - 1, 0, bz + eMin), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
                    eMin += x;
                }
            }

            //resolve Courtyard
            if (Rand.Range(0, 100) > 40)
            {
                ResolveParams parmsShed = parms;

                //shed
                parmsShed.rect = new CellRect(0, 0, 5, Rand.Range(4, 6));
                parmsShed.sketch = new Sketch();
                Sketch test = SketchGen.Generate(TROMSketchResolverDefOf.CourtyardShed, parmsShed);
                test.Rotate(Rot4.Random);
                test.Spawn(map, new IntVec3(bx + cye + Rand.Range(3, 9), 0, bz + cyn + Rand.Range(3, 9)), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: false, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: true);
            }
        }
    }

    class GenStep_TROM_Streets : GenStep
    {
        public override int SeedPart => 738902354;

        private TROMData tromdata = null;

        public override void Generate(Map map, GenStepParams parms)
        {
            tromdata = map.GetComponent<TROMData>();
            GenerateStreets(map, parms);
            GenerateIntersections(map, parms);
        }

        public void GenerateStreets(Map map, GenStepParams parms)
        {
            //horizontal streets
            int cx = -32;
            while (cx <= map.Size.x)
            {
                int cz = 6;
                while (cz <= map.Size.z)
                {
                    ResolveParams parms2 = default(ResolveParams);
                    parms2.sketch = new Sketch();
                    parms2.monumentSize = new IntVec2(38, 8);
                    SketchGen.Generate(TROMSketchResolverDefOf.Street, parms2).Spawn(map, new IntVec3(cx, 0, cz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: true, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: false);
                    cz += 46;
                }
                cx += 46;
            }

            //vertical streets
            cx = 6;
            while (cx <= map.Size.x)
            {
                int cz = -32;
                while (cz <= map.Size.z)
                {
                    ResolveParams parms2 = default(ResolveParams);
                    parms2.sketch = new Sketch();
                    parms2.monumentSize = new IntVec2(8, 38);
                    SketchGen.Generate(TROMSketchResolverDefOf.Street, parms2).Spawn(map, new IntVec3(cx, 0, cz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: true, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: false);
                    cz += 46;
                }
                cx += 46;
            }
        }

        public void GenerateIntersections(Map map, GenStepParams parms)
        {
            int cx = 6;
            while (cx <= map.Size.x)
            {
                int cz = 6;
                while (cz <= map.Size.z)
                {
                    ResolveParams parms2 = default(ResolveParams);
                    parms2.sketch = new Sketch();
                    parms2.monumentSize = new IntVec2(8, 8);
                    SketchGen.Generate(TROMSketchResolverDefOf.Intersection, parms2).Spawn(map, new IntVec3(cx, 0, cz), null, Sketch.SpawnPosType.Unchanged, Sketch.SpawnMode.Normal, wipeIfCollides: true, clearEdificeWhereFloor: false, dormant: false, buildRoofsInstantly: false);
                    cz += 46;
                }
                cx += 46;
            }
        }
    }
}