﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;

namespace TROM
{
    class SketchResolver_HouseCorner_R2 : SketchResolver
    {
        public static int x = 11, z = 11;
        bool switche = false;

        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            parms.rect = new CellRect(parms.rect.Value.minX, parms.rect.Value.minZ, x, z);
            ThingDef wallMaterial = Utility.Filters.RandStone();

            //old
            int width = parms.rect.Value.Width;
            int height = parms.rect.Value.Height;
            Sketch house = new Sketch();
            bool onlyBuildableByPlayer = parms.onlyBuildableByPlayer ?? false;
            bool filterAllowsAll = parms.allowedMonumentThings == null;

            bool[,] array = new bool[11, 11];
            bool[,] array1 = Utility.BoxShapeGenerator.Generate(width, 7);
            bool[,] array2 = Utility.BoxShapeGenerator.Generate(7, height);

            //overlay
            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < 7; j++)
                {
                    if (array1[i, j])
                    {
                        array[i, j + 4] = true;
                    }
                }
            }
            for (int i = 0; i < 7; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    if (array2[i, j])
                    {
                        array[i + 4, j] = true;
                    }
                }
            }

            //make space for doors
            array[6, 0] = false;
            array[0, 6] = false;
            array[4, 8] = false;
            array[8, 4] = false;

            //add walls
            for (int i = 0; i < array.GetLength(0); i++)
            {
                for (int j = 0; j < array.GetLength(1); j++)
                {
                    if (array[i, j])
                    {
                        house.AddThing(ThingDefOf.Wall, new IntVec3(i, 0, j), Rot4.North, wallMaterial);
                    }
                }
            }

            ResolveParams parms2 = parms;
            parms2.sketch = house;
            parms2.connectedGroupsSameStuff = true;
            if (parms.addFloors ?? true)
            {
                ResolveParams parms3 = parms;
                parms3.singleFloorType = true;
                parms3.sketch = house;
                parms3.floorFillRoomsOnly = false;
                parms3.onlyStoneFloors = parms.onlyStoneFloors ?? true;
                parms3.allowConcrete = parms.allowConcrete ?? false;
                parms3.rect = new CellRect(0, 0, width, height);
                SketchResolverDefOf.FloorFill.Resolve(parms3);
            }
            List<SketchThing> things = house.Things;
            for (int num6 = 0; num6 < things.Count; num6++)
            {
                if (things[num6].def == ThingDefOf.Wall)
                {
                    house.RemoveTerrain(things[num6].pos);
                }
            }

            //add doors
            house.AddThing(ThingDefOf.Door, new IntVec3(6, 0, 0), Rot4.North, ThingDefOf.WoodLog);
            house.AddThing(ThingDefOf.Door, new IntVec3(0, 0, 6), Rot4.East, ThingDefOf.WoodLog);
            house.AddThing(ThingDefOf.Door, new IntVec3(4, 0, 8), Rot4.East, ThingDefOf.WoodLog);
            house.AddThing(ThingDefOf.Door, new IntVec3(8, 0, 4), Rot4.North, ThingDefOf.WoodLog);


            //add furniture
            if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(5, 0, 3), Rot4.East);
            if (Rand.Range(0, 100) > 50) house.AddThing(ThingDefOf.PlantPot, new IntVec3(9, 0, 3), Rot4.North);
            if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(3, 0, 5), Rot4.North);
            if (Rand.Range(0, 100) > 60) house.AddThing(ThingDefOf.PlantPot, new IntVec3(3, 0, 9), Rot4.North);

            if (Rand.Range(0, 100) > 90) house.AddThing(TROMThingDefOf.ChessTable, new IntVec3(8, 0, 8), Rot4.North);
            if (Rand.Range(0, 100) > 60) house.AddThing(TROMThingDefOf.Armchair, new IntVec3(8, 0, 9), Rot4.South);
            if (Rand.Range(0, 100) > 60) house.AddThing(TROMThingDefOf.Armchair, new IntVec3(9, 0, 8), Rot4.West);
            if (Rand.Range(0, 100) > 95) house.AddThing(ThingDefOf.AncientLamp, new IntVec3(7, 0, 7), Rot4.North);
            if (Rand.Range(0, 100) > 50) house.AddThing(ThingDefOf.Table1x2c, new IntVec3(5, 0, 5), Rot4.North, ThingDefOf.WoodLog);
            if (Rand.Range(0, 100) > 20) house.AddThing(ThingDefOf.DiningChair, new IntVec3(6, 0, 5), Rot4.West, ThingDefOf.WoodLog);
            if (Rand.Range(0, 100) > 75) house.AddThing(ThingDefOf.DiningChair, new IntVec3(6, 0, 6), Rot4.West, ThingDefOf.WoodLog);
            if (Rand.Range(0, 100) > 75) house.AddThing(ThingDefOf.DiningChair, new IntVec3(5, 0, 7), Rot4.South, ThingDefOf.WoodLog);

            parms.sketch.MergeAt(house, new IntVec3(parms.rect.Value.minX, 0, parms.rect.Value.minZ), Sketch.SpawnPosType.Unchanged);
        }
    }
}
