﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;

namespace TROM
{
    class SketchResolver_HouseCorner_RG : SketchResolver
    {
        public static int x = 11, z = 7;
        bool switche = false;

        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            switche = (bool)parms.monumentOpen;
            ThingDef wallMaterial = Utility.Filters.RandStone();
            if (!switche)
            {
                parms.rect = new CellRect(parms.rect.Value.minX, parms.rect.Value.minZ, x, z);
            }
            else
            {
                parms.rect = new CellRect(parms.rect.Value.minX, parms.rect.Value.minZ, z, x);
            }

            Sketch house = new Sketch();
            int width = parms.rect.Value.Width;
            int height = parms.rect.Value.Height;

            bool onlyBuildableByPlayer = parms.onlyBuildableByPlayer ?? false;
            bool filterAllowsAll = parms.allowedMonumentThings == null;

            bool[,] array;

            //partition wall
            if (!switche)
            {
                array = Utility.BoxShapeGenerator.Generate(width, height);
                array[5, 1] = true;
                array[5, 2] = true;
                array[5, 3] = true;
                array[5, 4] = true;
                array[5, 5] = true;
            }
            else
            {
                array = Utility.BoxShapeGenerator.Generate(width, height);
                array[1, 5] = true;
                array[2, 5] = true;
                array[3, 5] = true;
                array[4, 5] = true;
                array[5, 5] = true;
            }

            //make space for doors
            if (!switche)
            {
                array[0, 3] = false;
                array[8, 0] = false;
            }
            else
            {
                array[3, 0] = false;
                array[0, 8] = false;
            }

            //add walls
            for (int i = 0; i < array.GetLength(0); i++)
            {
                for (int j = 0; j < array.GetLength(1); j++)
                {
                    if (array[i, j])
                    {
                        house.AddThing(ThingDefOf.Wall, new IntVec3(i, 0, j), Rot4.North, wallMaterial);
                    }
                }
            }

            ResolveParams parms2 = parms;
            parms2.sketch = house;
            parms2.connectedGroupsSameStuff = true;
            if (parms.addFloors ?? true)
            {
                ResolveParams parms3 = parms;
                parms3.singleFloorType = true;
                parms3.sketch = house;
                parms3.floorFillRoomsOnly = false;
                parms3.onlyStoneFloors = parms.onlyStoneFloors ?? true;
                parms3.allowConcrete = parms.allowConcrete ?? false;
                parms3.rect = new CellRect(0, 0, width, height);
                SketchResolverDefOf.FloorFill.Resolve(parms3);
            }
            List<SketchThing> things = house.Things;
            for (int num6 = 0; num6 < things.Count; num6++)
            {
                if (things[num6].def == ThingDefOf.Wall)
                {
                    house.RemoveTerrain(things[num6].pos);
                }
            }

            //add doors
            if (!switche)
            {
                house.AddThing(ThingDefOf.Door, new IntVec3(0, 0, 3), Rot4.East, ThingDefOf.WoodLog);
                house.AddThing(ThingDefOf.Door, new IntVec3(8, 0, 0), Rot4.North, ThingDefOf.WoodLog);
            }
            else
            {
                house.AddThing(ThingDefOf.Door, new IntVec3(3, 0, 0), Rot4.North, ThingDefOf.WoodLog);
                house.AddThing(ThingDefOf.Door, new IntVec3(0, 0, 8), Rot4.East, ThingDefOf.WoodLog);
            }
            
            //add furniture
            if (!switche)
            {
                if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(4, 0, 1), Rot4.North);
                if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(9, 0, 5), Rot4.West);
            }
            else
            {
                if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(5, 0, 4), Rot4.West);
                if (Rand.Range(0, 100) > 30) house.AddThing(ThingDefOf.AncientBed, new IntVec3(5, 0, 9), Rot4.West);
            }

            parms.sketch.MergeAt(house, new IntVec3(parms.rect.Value.minX, 0, parms.rect.Value.minZ), Sketch.SpawnPosType.Unchanged);
        }
    }
}
