﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using Verse;

namespace TROM
{
    class SketchResolver_HouseEdge : SketchResolver
    {
        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            ThingDef wallMaterial = Utility.Filters.RandStone();

            int width = parms.rect.Value.Width;
            int height = parms.rect.Value.Height;
            int door = Rand.Range(1, width - 2);
            Sketch house = new Sketch();
            bool onlyBuildableByPlayer = parms.onlyBuildableByPlayer ?? false;
            bool filterAllowsAll = parms.allowedMonumentThings == null;

            bool[,] array = Utility.BoxShapeGenerator.Generate(width, height);
            array[door, 0] = false;
            for (int i = 0; i < array.GetLength(0); i++)
            {
                for (int j = 0; j < array.GetLength(1); j++)
                {
                    if (array[i, j])
                    {
                        house.AddThing(ThingDefOf.Wall, new IntVec3(i, 0, j), Rot4.North, wallMaterial);
                    }
                }
            }

            ResolveParams parms2 = parms;
            parms2.sketch = house;
            parms2.connectedGroupsSameStuff = true;
            if (parms.addFloors ?? true)
            {
                ResolveParams parms3 = parms;
                parms3.thingCentral = ThingDefOf.WoodLog;
                parms3.singleFloorType = true;
                parms3.sketch = house;
                parms3.floorFillRoomsOnly = false;
                parms3.onlyStoneFloors = parms.onlyStoneFloors ?? false;
                parms3.allowConcrete = parms.allowConcrete ?? false;
                parms3.rect = new CellRect(1, 1, width - 2, height - 2);
                TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
                parms3.rect = new CellRect(door, 0, 1, 1);
                parms3.thingCentral = wallMaterial;
                TROMSketchResolverDefOf.FloorFillSpecific.Resolve(parms3);
            }
            List<SketchThing> things = house.Things;
            for (int num6 = 0; num6 < things.Count; num6++)
            {
                if (things[num6].def == ThingDefOf.Wall)
                {
                    house.RemoveTerrain(things[num6].pos);
                }
            }

            house.AddThing(ThingDefOf.Door, new IntVec3(door, 0, 0), Rot4.North);

            //debris
            if (Rand.Bool)
            {
                house.AddThing(ThingDefOf.AncientBed, new IntVec3(Rand.Range(1, width - 2), 0, height - 3), Rot4.North);
            }

            parms.sketch.MergeAt(house, new IntVec3(parms.rect.Value.minX, 0, parms.rect.Value.minZ), Sketch.SpawnPosType.Unchanged);
        }
    }
}
