﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;

namespace TROM
{
    class SketchResolver_Intersection : SketchResolver
    {
        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            ResolveParams parms2 = parms;
            IntVec2 intVec = parms.monumentSize.Value;
            int width = intVec.x;
            int height = intVec.z;
            Sketch intersection = new Sketch();

            //shape generator
            bool[,] array0 = Utility.IntersectionShapeGenerator.Generate(width);

            //intersection
            for (int i = 0; i < array0.GetLength(0); i++)
            {
                for (int j = 0; j < array0.GetLength(1); j++)
                {
                    if (array0[i, j])
                    {
                        intersection.AddTerrain(TerrainDefOf.PavedTile, new IntVec3(i, 0, j), false);
                    }
                    else
                    {
                        intersection.AddTerrain(TROMTerrainDefOf.BrokenAsphalt, new IntVec3(i, 0, j), false);
                    }
                }
            }
            parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);


            intersection = new Sketch();
            //shape generator
            bool[,] array1 = new bool[width, height];
            for (int i = 0; i < width; i++)
            {
                for (int j = 0; j < height; j++)
                {
                    if ((i == 1 || i == 6) && (j == 1 || j == 6))
                    {
                        array1[i, j] = true;
                    }
                    else
                    {
                        array1[i, j] = false;
                    }
                }
            }

            //set AncientLampposts
            for (int i = 0; i < array1.GetLength(0); i++)
            {
                for (int j = 0; j < array1.GetLength(1); j++)
                {
                    if (array1[i, j])
                    {
                        if (Rand.Range(0, 100) > 20)
                        {
                            intersection.AddThing(TROMThingDefOf.AncientLamppost, new IntVec3(i, 0, j), Rot4.North);
                        }
                    }
                }
            }
            parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);

            int num = Rand.Range(0, 100);
            if (num <= 30)//empty
            {
                
            }
            else if (30 < num && num <= 70)//destroyed cars
            {
                intersection = new Sketch();
                intersection.AddThing(ThingDefOf.AncientRustedCar, new IntVec3(Rand.Range(2, 6), 0, Rand.Range(2, 6)), Rot4.Random);
                parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);

            }
            else if (70 < num && num <= 90)//barbed wire
            {
                //shape generator
                bool[,] array2 = new bool[width, height];
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < height; j++)
                    {
                        if (((i == 1 || i == 6) && (j >= 2 && j <= 5)) || ((j == 1 || j == 6) && (i >= 2 && i <= 5)))
                        {
                            array2[i, j] = true;
                        }
                        else
                        {
                            array2[i, j] = false;
                        }
                    }
                }

                //set AncientRazorWire
                intersection = new Sketch();
                for (int i = 0; i < array2.GetLength(0); i++)
                {
                    for (int j = 0; j < array2.GetLength(1); j++)
                    {
                        if (array2[i, j])
                        {
                            if (Rand.Range(0, 100) > 20)
                            {
                                if (i == 1 || i == 6)
                                {
                                    intersection.AddThing(ThingDefOf.AncientRazorWire, new IntVec3(i, 0, j), Rot4.East);
                                }
                                if (j == 1 || j == 6)
                                {
                                    intersection.AddThing(ThingDefOf.AncientRazorWire, new IntVec3(i, 0, j), Rot4.North);
                                }
                            }
                        }
                    }
                }
                parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);

                //set AncientSecurityTurret
                intersection = new Sketch();
                intersection.AddThing(ThingDefOf.AncientSecurityTurret, new IntVec3(Rand.Range(2, 6), 0, Rand.Range(2, 6)), Rot4.Random);
                parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);
            }
            else if (90 < num && num <= 96)//artillery
            {
                //shape generator
                bool[,] array3 = new bool[width, height];
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < height; j++)
                    {
                        if (((i == 0 || i == 7) && (j >= 2 && j <= 5)) || ((j == 0 || j == 7) && (i >= 2 && i <= 5)))
                        {
                            array3[i, j] = true;
                        }
                        else
                        {
                            array3[i, j] = false;
                        }
                    }
                }

                //set AncientRazorWire
                intersection = new Sketch();
                for (int i = 0; i < array3.GetLength(0); i++)
                {
                    for (int j = 0; j < array3.GetLength(1); j++)
                    {
                        if (array3[i, j])
                        {
                            if (Rand.Range(0, 100) > 20)
                            {
                                if (i == 0 || i == 7)
                                {
                                    intersection.AddThing(ThingDefOf.AncientRazorWire, new IntVec3(i, 0, j), Rot4.East);
                                }
                                if (j == 0 || j == 7)
                                {
                                    intersection.AddThing(ThingDefOf.AncientRazorWire, new IntVec3(i, 0, j), Rot4.North);
                                }
                            }
                        }
                    }
                }
                parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);

                //set AncientMegaCannonTripod
                intersection = new Sketch();
                intersection.AddThing(ThingDefOf.AncientMegaCannonTripod, new IntVec3(4, 0, 4), Rot4.Random);
                parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);

                //set AncientMegaCannonTripod
                if (Rand.Range(0, 100) > 50)
                {
                    intersection = new Sketch();
                    intersection.AddThing(ThingDefOf.AncientMegaCannonBarrel, new IntVec3(2, 0, 2), Rot4.Random);
                    parms.sketch.MergeAt(intersection, default(IntVec3), Sketch.SpawnPosType.Unchanged);
                }
            }
            else if (96 < num && num <= 100)//checkpoint
            {
                intersection = new Sketch();
                parms.rect = new CellRect(0, 0, width, height);
                TROMSketchResolverDefOf.Intersection_Checkpoint.Resolve(parms);
            }
        }
    }
}
