﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;

namespace TROM
{
    class SketchResolver_Street : SketchResolver
    {
        protected override bool CanResolveInt(ResolveParams parms)
        {
            return true;
        }

        protected override void ResolveInt(ResolveParams parms)
        {
            ResolveParams parms2 = parms;
            IntVec2 intVec = parms.monumentSize.Value;
            int width = intVec.x;
            int height = intVec.z;
            int num = Rand.Range(0, 100);
            Sketch street = new Sketch();

            //horizontal street
            if (width > height)
            {
                //street
                bool[,] array = Utility.StreetShapeGenerator.Generate(width, height);
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if (array[i, j])
                        {
                            street.AddTerrain(TerrainDefOf.PavedTile, new IntVec3(i, 0, j), false);
                        }
                        else
                        {
                            street.AddTerrain(TROMTerrainDefOf.BrokenAsphalt, new IntVec3(i, 0, j), false);
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));
                //AncientLampposts
                array = new bool[width,height];
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if ((j == 1 || j == 6) && (i == 6 || i == 14 || i == 23 || i == 31))
                        {
                            array[i, j] = true;
                        }
                        else
                        {
                            array[i, j] = false;
                        }
                    }
                }
                street = new Sketch();
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if (array[i, j] && Rand.Range(0, 100) > 20)
                        {
                            street.AddThing(TROMThingDefOf.AncientLamppost, new IntVec3(i, 0, j), Rot4.North);
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));

                //AncientRustedCar
                street = new Sketch();
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < height; j++)
                    {
                        if (j == 3 && (j > 1 || j < width - 2))
                        {
                            if (Rand.Range(0, 100) > 92)
                            {
                                street.AddThing(ThingDefOf.AncientRustedCar, new IntVec3(i, 0, j), Rot4.East);
                            }
                        }
                        if (j == 4 && (j > 2 || j < width - 1))
                        {
                            if (Rand.Range(0, 100) > 92)
                            {
                                street.AddThing(ThingDefOf.AncientRustedCar, new IntVec3(i, 0, j), Rot4.West);
                            }
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));

                //barricade 7 cells
                if (num >= 98)
                {
                    //orientation
                    if (Rand.Bool)
                    {
                        //left
                        parms.rect = new CellRect(Rand.Range(1, 5), 0, 0, 0);
                        parms.SetCustom("direction", Rot4.East);
                        TROMSketchResolverDefOf.Street_Barricade.Resolve(parms);
                    }
                    else
                    {
                        //right
                        parms.rect = new CellRect(Rand.Range(26, 31), 0, 0, 0);
                        parms.SetCustom("direction", Rot4.West);
                        TROMSketchResolverDefOf.Street_Barricade.Resolve(parms);
                    }
                }
            }

            //vertical street
            else
            {
                //street
                bool[,] array = Utility.StreetShapeGenerator.Generate(width, height);
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if (array[i, j])
                        {
                            street.AddTerrain(TerrainDefOf.PavedTile, new IntVec3(i, 0, j), false);
                        }
                        else
                        {
                            street.AddTerrain(TROMTerrainDefOf.BrokenAsphalt, new IntVec3(i, 0, j), false);
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));
                //AncientLampposts
                array = new bool[width, height];
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if ((i == 1 || i == 6) && (j == 6 || j == 14 || j == 23 || j == 31))
                        {
                            array[i, j] = true;
                        }
                        else
                        {
                            array[i, j] = false;
                        }
                    }
                }
                street = new Sketch();
                for (int i = 0; i < array.GetLength(0); i++)
                {
                    for (int j = 0; j < array.GetLength(1); j++)
                    {
                        if (array[i, j] && Rand.Range(0, 100) > 20)
                        {
                            street.AddThing(TROMThingDefOf.AncientLamppost, new IntVec3(i, 0, j), Rot4.North);
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));

                //AncientRustedCar
                street = new Sketch();
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < height; j++)
                    {
                        if (i == 3 && (j > 1 || j < height-2))
                        {
                            if (Rand.Range(0, 100) > 95)
                            {
                                street.AddThing(ThingDefOf.AncientRustedCar, new IntVec3(i, 0, j), Rot4.South);
                            }
                        }
                        if (i == 4 && (j > 2 || j < height - 1))
                        {
                            if (Rand.Range(0, 100) > 95)
                            {
                                street.AddThing(ThingDefOf.AncientRustedCar, new IntVec3(i, 0, j), Rot4.North);
                            }
                        }
                    }
                }
                parms.sketch.MergeAt(street, default(IntVec3));

                //barricade 7 cells
                if (num >= 98)
                {
                    //orientation
                    if (Rand.Bool)
                    {
                        //up
                        parms.rect = new CellRect(0, Rand.Range(26, 31), 0, 0);
                        parms.SetCustom("direction", Rot4.South);
                        TROMSketchResolverDefOf.Street_Barricade.Resolve(parms);
                        
                    }
                    else
                    {
                        //down
                        parms.rect = new CellRect(0, Rand.Range(1, 5), 0, 0);
                        parms.SetCustom("direction", Rot4.North);
                        TROMSketchResolverDefOf.Street_Barricade.Resolve(parms);
                    }
                }
            }
        }
    }
}
