﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using RimWorld.SketchGen;
using UnityEngine;
using Verse;

namespace TROM
{
    namespace Utility
    {
        public static class BoxShapeGenerator
        {
            public static bool[,] Generate(int width, int height)
            {
                bool[,] array = new bool[width, height];
                for (int i = 0; i < height; i++)
                {
                    for (int j = 0; j < width; j++)
                    {
                        if ((i == 0 || j == 0) || (i == height-1 || j == width-1))
                        {
                            array[j, i] = true;
                        }
                    }
                }
                return array;
            }
        }

        public static class LBoxShapeGenerator
        {
            public static bool[,] Generate(int width, int height, int thickness)
            {
                bool[,] array = new bool[width, height];
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < height; j++)
                    {
                        if ((i == 0 || j == 0) || (i == width - 1 && j < thickness ) || (j == height - 1 && i < thickness) || (i == thickness - 1 && j >= thickness - 1) || (j == thickness - 1 && i >= thickness - 1))
                        {
                            array[i, j] = true;
                        }
                    }
                }
                return array;
            }
        }

        public static class StreetShapeGenerator
        {
            public static bool[,] Generate(int width, int height)
            {
                bool[,] array = new bool[width, height];
                if (width > height)
                {
                    for (int i = 0; i < width; i++)
                    {
                        for (int j = 0; j < height; j++)
                        {
                            if ((j < 2) || (j > 5))
                            {
                                array[i, j] = true;
                            }
                            else
                            {
                                array[i, j] = false;
                            }
                        }
                    }
                }
                else
                {
                    for (int i = 0; i < width; i++)
                    {
                        for (int j = 0; j < height; j++)
                        {
                            if ((i < 2) || (i > 5))
                            {
                                array[i, j] = true;
                            }
                            else
                            {
                                array[i, j] = false;
                            }
                        }
                    }
                }
                return array;
            }
        }

        public static class IntersectionShapeGenerator
        {
            public static bool[,] Generate(int width)
            {
                bool[,] array = new bool[width, width];
                for (int i = 0; i < width; i++)
                {
                    for (int j = 0; j < width; j++)
                    {
                        if ((i < 2 || i > 5) && (j < 2 || j > 5))
                        {
                            array[i, j] = true;
                        }
                        else
                        {
                            array[i, j] = false;
                        }
                    }
                }
                return array;
            }
        }

        public static class Filters
        {
            public static ThingDef RandStone()
            {
                int x = Rand.Range(0, 100);
                if (x <= 20) return ThingDefOf.BlocksGranite;
                if (x <= 40) return TROMThingDefOf.BlocksLimestone;
                if (x <= 60) return TROMThingDefOf.BlocksMarble;
                if (x <= 80) return TROMThingDefOf.BlocksSandstone;
                return TROMThingDefOf.BlocksSlate;
            }
        }
    }
}
