﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using RimWorld;
using RimWorld.Planet;
using Verse;
using Verse.Noise;

namespace TROM
{
    class WorldGenStep_TROM_City : WorldGenStep
    {
        public override int SeedPart => 349793029;

        private List<int> cities = new List<int>();

        public override void GenerateFresh(string seed)
        {
            GenerateCityCentral();
        }

        public int FindCityCentralTiles()
        {
            int xx = 0;
            //limiter
            for (int i = 0; i < 500; i++)
            {
                //?
                if ((from _ in Enumerable.Range(0, 100)
                     select Rand.Range(0, Find.WorldGrid.TilesCount)).TryRandomElementByWeight(delegate (int x)
                     {
                         Tile tile = Find.WorldGrid[x];
                         //biome check
                         if (!tile.biome.canBuildBase || !tile.biome.implemented)
                         {
                             return 0f;
                         }
                         //only on flat tiles
                         if (!(tile.hilliness == Hilliness.Flat))
                         {
                             return 0f;
                         }
                         //?
                         if (!tile.biome.canAutoChoose)
                         {
                             return 0f;
                         }
                         //river/road check
                         if (!tile.potentialRivers.NullOrEmpty() || !tile.potentialRoads.NullOrEmpty())
                         {
                             return 0f;
                         }
                         float num = tile.biome.settlementSelectionWeight;
                         return num;
                     }, out var result) && IsValidTileForNewCity(result))
                {
                    return result;
                }
            }
            return 0;
        }

        public bool IsValidTileForNewCity(int tile)
        {
            //if first city
            if (cities.Count == 0)
            {
                return true;
            }
            //no other city nearby ~10 tiles
            for (int i = 0; i < cities.Count; i++)
            {
                if (Find.WorldGrid.ApproxDistanceInTiles(tile, cities[i]) < 10f)
                {
                    return false;
                }
            }
            return true;
        }

        public void GenerateCityCentral()
        {
            List<Tile> tiles = Find.WorldGrid.tiles;
            int iterationLimit = (int) (1000 * Find.World.info.planetCoverage);
            int cityLimit = (int)(100 * Find.World.info.planetCoverage);
            int cityCount = 0;
            for (int i = 0; (i < iterationLimit) && (cityCount < cityLimit); i++)
            {
                //make city if valid params
                int tileId = FindCityCentralTiles();
                Tile tile = tiles[tileId];

                if (tile.temperature > -0f && tile.temperature < 25f)
                {
                    if (tile.rainfall > 400f && tile.rainfall < 2000f)
                    {
                        if (tile.elevation < 1000f && tile.elevation > 20f)
                        {

                            tile.biome = TROMBiomeDefOf.CityCentral;
                            cities.Add(tileId);
                            GenerateCityArea(tileId);
                            cityCount++;
                        }
                    }
                }
            }
        }

        public void GenerateCityArea(int tile)
        {
            List<int> adjacent = new List<int>();
            Find.WorldGrid.GetTileNeighbors(tile, adjacent);
            for (int i = 0; i < adjacent.Count; i++)
            {
                Find.WorldGrid.tiles[adjacent[i]].biome = TROMBiomeDefOf.CityOutskirts;
            }
        }
    }
}
